#ifndef ASCXX_SOLVER_PARAMETER_H
#define ASCXX_SOLVER_PARAMETER_H

#include <string>
#include <vector>

#include "config.h"
extern "C"{
#include "ascend/system/slv_param.h"
}

#include "value.h"

class SolverParameter{
private:
	struct slv_parameter *p;

public:
	explicit SolverParameter(slv_parameter *);
	SolverParameter &operator=(const SolverParameter &);

	const std::string getName() const;
	const std::string getDescription() const;
	const std::string getLabel() const;
	const int &getNumber() const;
	const int &getPage() const;

	const bool isInt() const;
	const bool isBool() const;
	const bool isStr() const;
	const bool isReal() const;

	// The following throw execeptions unless the parameter type is correct
	const int &getIntValue() const;
	const int &getIntLowerBound() const;
	const int &getIntUpperBound() const;
	void setIntValue(const int&);

	const bool getBoolValue() const;
	void setBoolValue(const bool&);

	const std::string getStrValue() const;
	const std::vector<std::string> getStrOptions() const;
	void setStrValue(const std::string &);
	void setStrOption(const int &opt);

	const double &getRealValue() const;
	const double &getRealLowerBound() const;
	const double &getRealUpperBound() const;
	void setRealValue(const double&);

	const bool isBounded() const;

	const std::string toString() const;

	void setValueValue(const Value &V);
};

#endif
